@@ -1,4 +1,4 @@
-// Copyright 2018-2021 Parity Technologies (UK) Ltd.
+// Copyright 2018-2022 Parity Technologies (UK) Ltd.
 // This file is part of cargo-contract.
 //
 // cargo-contract is free software: you can redistribute it and/or modify
@@ -20,48 +20,46 @@ mod util;
 mod validate_wasm;
 mod workspace;
 
-use self::workspace::ManifestPath;
+use self::{
+    cmd::{
+        metadata::MetadataResult, BuildCommand, CallCommand, CheckCommand, InstantiateCommand,
+        TestCommand, UploadCommand,
+    },
+    util::DEFAULT_KEY_COL_WIDTH,
+    workspace::ManifestPath,
+};
 
-use crate::cmd::{metadata::MetadataResult, BuildCommand, CheckCommand, TestCommand};
-
-#[cfg(feature = "extrinsics")]
-use sp_core::{crypto::Pair, sr25519, H256};
 use std::{
     convert::TryFrom,
     fmt::{Display, Formatter, Result as DisplayResult},
     path::PathBuf,
     str::FromStr,
 };
-#[cfg(feature = "extrinsics")]
-use subxt::PairSigner;
 
 use anyhow::{Error, Result};
+use clap::{AppSettings, Args, Parser, Subcommand};
 use colored::Colorize;
-use structopt::{clap, StructOpt};
 
-#[derive(Debug, StructOpt)]
-#[structopt(bin_name = "cargo")]
-#[structopt(version = env!("CARGO_CONTRACT_CLI_IMPL_VERSION"))]
+#[derive(Debug, Parser)]
+#[clap(bin_name = "cargo")]
+#[clap(version = env!("CARGO_CONTRACT_CLI_IMPL_VERSION"))]
 pub(crate) enum Opts {
     /// Utilities to develop Wasm smart contracts.
-    #[structopt(name = "contract")]
-    #[structopt(version = env!("CARGO_CONTRACT_CLI_IMPL_VERSION"))]
-    #[structopt(setting = clap::AppSettings::UnifiedHelpMessage)]
-    #[structopt(setting = clap::AppSettings::DeriveDisplayOrder)]
-    #[structopt(setting = clap::AppSettings::DontCollapseArgsInUsage)]
+    #[clap(name = "contract")]
+    #[clap(version = env!("CARGO_CONTRACT_CLI_IMPL_VERSION"))]
+    #[clap(setting = AppSettings::DeriveDisplayOrder)]
     Contract(ContractArgs),
 }
 
-#[derive(Debug, StructOpt)]
+#[derive(Debug, Args)]
 pub(crate) struct ContractArgs {
-    #[structopt(subcommand)]
+    #[clap(subcommand)]
     cmd: Command,
 }
 
 #[derive(Debug, Default, Clone, PartialEq, Eq)]
 pub(crate) struct HexData(pub Vec<u8>);
 
-#[cfg(feature = "extrinsics")]
 impl std::str::FromStr for HexData {
     type Err = hex::FromHexError;
 
@@ -70,36 +68,6 @@ impl std::str::FromStr for HexData {
     }
 }
 
-/// Arguments required for creating and sending an extrinsic to a substrate node
-#[cfg(feature = "extrinsics")]
-#[derive(Debug, StructOpt)]
-pub(crate) struct ExtrinsicOpts {
-    /// Websockets url of a substrate node
-    #[structopt(
-        name = "url",
-        long,
-        parse(try_from_str),
-        default_value = "ws://localhost:9944"
-    )]
-    url: url::Url,
-    /// Secret key URI for the account deploying the contract.
-    #[structopt(name = "suri", long, short)]
-    suri: String,
-    /// Password for the secret key
-    #[structopt(name = "password", long, short)]
-    password: Option<String>,
-}
-
-#[cfg(feature = "extrinsics")]
-impl ExtrinsicOpts {
-    pub fn signer(&self) -> Result<PairSigner<subxt::DefaultNodeRuntime, sr25519::Pair>> {
-        let pair =
-            sr25519::Pair::from_string(&self.suri, self.password.as_ref().map(String::as_ref))
-                .map_err(|_| anyhow::anyhow!("Secret string error"))?;
-        Ok(PairSigner::new(pair))
-    }
-}
-
 #[derive(Clone, Copy, Debug, PartialEq)]
 pub enum OptimizationPasses {
     Zero,
@@ -139,7 +107,7 @@ impl std::str::FromStr for OptimizationPasses {
         // We need to replace " here, since the input string could come
         // from either the CLI or the `Cargo.toml` profile section.
         // If it is from the profile it could e.g. be "3" or 3.
-        let normalized_input = input.replace("\"", "").to_lowercase();
+        let normalized_input = input.replace('"', "").to_lowercase();
         match normalized_input.as_str() {
             "0" => Ok(OptimizationPasses::Zero),
             "1" => Ok(OptimizationPasses::One),
@@ -159,18 +127,18 @@ impl From<std::string::String> for OptimizationPasses {
     }
 }
 
-#[derive(Default, Clone, Debug, StructOpt)]
+#[derive(Default, Clone, Debug, Args)]
 pub struct VerbosityFlags {
     /// No output printed to stdout
-    #[structopt(long)]
+    #[clap(long)]
     quiet: bool,
     /// Use verbose output
-    #[structopt(long)]
+    #[clap(long)]
     verbose: bool,
 }
 
 /// Denotes if output should be printed to stdout.
-#[derive(Clone, Copy, serde::Serialize)]
+#[derive(Clone, Copy, serde::Serialize, Eq, PartialEq)]
 pub enum Verbosity {
     /// Use default output
     Default,
@@ -209,10 +177,10 @@ impl TryFrom<&VerbosityFlags> for Verbosity {
     }
 }
 
-#[derive(Default, Clone, Debug, StructOpt)]
+#[derive(Default, Clone, Debug, Args)]
 struct UnstableOptions {
     /// Use the original manifest (Cargo.toml), do not modify for build optimizations
-    #[structopt(long = "unstable-options", short = "Z", number_of_values = 1)]
+    #[clap(long = "unstable-options", short = 'Z', number_of_values = 1)]
     options: Vec<String>,
 }
 
@@ -241,14 +209,14 @@ impl TryFrom<&UnstableOptions> for UnstableFlags {
 }
 
 /// Describes which artifacts to generate
-#[derive(Copy, Clone, Eq, PartialEq, Debug, StructOpt, serde::Serialize)]
-#[structopt(name = "build-artifacts")]
+#[derive(Copy, Clone, Eq, PartialEq, Debug, Subcommand, serde::Serialize)]
+#[clap(name = "build-artifacts")]
 pub enum BuildArtifacts {
     /// Generate the Wasm, the metadata and a bundled `<name>.contract` file
-    #[structopt(name = "all")]
+    #[clap(name = "all")]
     All,
     /// Only the Wasm is created, generation of metadata and a bundled `<name>.contract` file is skipped
-    #[structopt(name = "code-only")]
+    #[clap(name = "code-only")]
     CodeOnly,
     CheckOnly,
 }
@@ -369,7 +337,7 @@ pub struct BuildResult {
 /// Result of the optimization process.
 #[derive(serde::Serialize)]
 pub struct OptimizationResult {
-    /// The path of the optimized wasm file.
+    /// The path of the optimized Wasm file.
     pub dest_wasm: PathBuf,
     /// The original Wasm size.
     pub original_size: f64,
@@ -457,78 +425,43 @@ impl BuildResult {
     }
 }
 
-#[derive(Debug, StructOpt)]
+#[derive(Debug, Subcommand)]
 enum Command {
     /// Setup and create a new smart contract project
-    #[structopt(name = "new")]
+    #[clap(name = "new")]
     New {
         /// The name of the newly created smart contract
         name: String,
         /// The optional target directory for the contract project
-        #[structopt(short, long, parse(from_os_str))]
+        #[clap(short, long, parse(from_os_str))]
         target_dir: Option<PathBuf>,
     },
     /// Compiles the contract, generates metadata, bundles both together in a `<name>.contract` file
-    #[structopt(name = "build")]
+    #[clap(name = "build")]
     Build(BuildCommand),
     /// Check that the code builds as Wasm; does not output any `<name>.contract` artifact to the `target/` directory
-    #[structopt(name = "check")]
+    #[clap(name = "check")]
     Check(CheckCommand),
     /// Test the smart contract off-chain
-    #[structopt(name = "test")]
+    #[clap(name = "test")]
     Test(TestCommand),
-    /// Upload the smart contract code to the chain
-    #[cfg(feature = "extrinsics")]
-    #[structopt(name = "deploy")]
-    Deploy {
-        #[structopt(flatten)]
-        extrinsic_opts: ExtrinsicOpts,
-        /// Path to wasm contract code, defaults to `./target/ink/<name>.wasm`
-        #[structopt(parse(from_os_str))]
-        wasm_path: Option<PathBuf>,
-    },
-    /// Instantiate a deployed smart contract
-    #[cfg(feature = "extrinsics")]
-    #[structopt(name = "instantiate")]
-    Instantiate {
-        #[structopt(flatten)]
-        extrinsic_opts: ExtrinsicOpts,
-        /// Transfers an initial balance to the instantiated contract
-        #[structopt(name = "endowment", long, default_value = "0")]
-        endowment: u128,
-        /// Maximum amount of gas to be used for this command
-        #[structopt(name = "gas", long, default_value = "500000000")]
-        gas_limit: u64,
-        /// The hash of the smart contract code already uploaded to the chain
-        #[structopt(long, parse(try_from_str = parse_code_hash))]
-        code_hash: H256,
-        /// Hex encoded data to call a contract constructor
-        #[structopt(long)]
-        data: HexData,
-    },
-}
-
-#[cfg(feature = "extrinsics")]
-fn parse_code_hash(input: &str) -> Result<H256> {
-    let bytes = hex::decode(input)?;
-    if bytes.len() != 32 {
-        anyhow::bail!("Code hash should be 32 bytes in length")
-    }
-    let mut arr = [0u8; 32];
-    arr.copy_from_slice(&bytes);
-    Ok(H256(arr))
+    /// Upload contract code
+    #[clap(name = "upload")]
+    Upload(UploadCommand),
+    /// Instantiate a contract
+    #[clap(name = "instantiate")]
+    Instantiate(InstantiateCommand),
+    /// Call a contract
+    #[clap(name = "call")]
+    Call(CallCommand),
 }
 
 fn main() {
     env_logger::init();
 
-    let Opts::Contract(args) = Opts::from_args();
+    let Opts::Contract(args) = Opts::parse();
     match exec(args.cmd) {
-        Ok(maybe_msg) => {
-            if let Some(msg) = maybe_msg {
-                println!("\t{}", msg)
-            }
-        }
+        Ok(()) => {}
         Err(err) => {
             eprintln!(
                 "{} {}",
@@ -540,19 +473,22 @@ fn main() {
     }
 }
 
-fn exec(cmd: Command) -> Result<Option<String>> {
+fn exec(cmd: Command) -> Result<()> {
     match &cmd {
-        Command::New { name, target_dir } => cmd::new::execute(name, target_dir.as_ref()),
+        Command::New { name, target_dir } => {
+            cmd::new::execute(name, target_dir.as_ref())?;
+            println!("Created contract {}", name);
+            Ok(())
+        }
         Command::Build(build) => {
             let result = build.exec()?;
 
             if matches!(result.output_type, OutputType::Json) {
-                Ok(Some(result.serialize_json()?))
+                println!("{}", result.serialize_json()?)
             } else if result.verbosity.is_verbose() {
-                Ok(Some(result.display()))
-            } else {
-                Ok(None)
+                println!("{}", result.display())
             }
+            Ok(())
         }
         Command::Check(check) => {
             let res = check.exec()?;
@@ -561,46 +497,20 @@ fn exec(cmd: Command) -> Result<Option<String>> {
                 "no dest_wasm must be on the generation result"
             );
             if res.verbosity.is_verbose() {
-                Ok(Some(
-                    "\nYour contract's code was built successfully.".to_string(),
-                ))
-            } else {
-                Ok(None)
+                println!("\nYour contract's code was built successfully.")
             }
+            Ok(())
         }
         Command::Test(test) => {
             let res = test.exec()?;
             if res.verbosity.is_verbose() {
-                Ok(Some(res.display()?))
-            } else {
-                Ok(None)
+                println!("{}", res.display()?)
             }
+            Ok(())
         }
-        #[cfg(feature = "extrinsics")]
-        Command::Deploy {
-            extrinsic_opts,
-            wasm_path,
-        } => {
-            let code_hash = cmd::execute_deploy(extrinsic_opts, wasm_path.as_ref())?;
-            Ok(Some(format!("Code hash: {:?}", code_hash)))
-        }
-        #[cfg(feature = "extrinsics")]
-        Command::Instantiate {
-            extrinsic_opts,
-            endowment,
-            code_hash,
-            gas_limit,
-            data,
-        } => {
-            let contract_account = cmd::execute_instantiate(
-                extrinsic_opts,
-                *endowment,
-                *gas_limit,
-                *code_hash,
-                data.clone(),
-            )?;
-            Ok(Some(format!("Contract account: {:?}", contract_account)))
-        }
+        Command::Upload(upload) => upload.run(),
+        Command::Instantiate(instantiate) => instantiate.run(),
+        Command::Call(call) => call.run(),
     }
 }
 
