@@ -1,4 +1,4 @@
-// Copyright 2018-2021 Parity Technologies (UK) Ltd.
+// Copyright 2018-2022 Parity Technologies (UK) Ltd.
 // This file is part of cargo-contract.
 //
 // cargo-contract is free software: you can redistribute it and/or modify
@@ -31,14 +31,14 @@ const MANIFEST_FILE: &str = "Cargo.toml";
 const LEGACY_METADATA_PACKAGE_PATH: &str = ".ink/abi_gen";
 const METADATA_PACKAGE_PATH: &str = ".ink/metadata_gen";
 
-/// Path to a Cargo.toml file
+/// Path to a `Cargo.toml` file
 #[derive(Clone, Debug)]
 pub struct ManifestPath {
     path: PathBuf,
 }
 
 impl ManifestPath {
-    /// Create a new ManifestPath, errors if not path to `Cargo.toml`
+    /// Create a new [`ManifestPath`], errors if not path to `Cargo.toml`
     pub fn new<P: AsRef<Path>>(path: P) -> Result<Self> {
         let manifest = path.as_ref();
         if let Some(file_name) = manifest.file_name() {
@@ -52,8 +52,12 @@ impl ManifestPath {
     }
 
     /// Create an arg `--manifest-path=` for `cargo` command
-    pub fn cargo_arg(&self) -> String {
-        format!("--manifest-path={}", self.path.to_string_lossy())
+    pub fn cargo_arg(&self) -> Result<String> {
+        let path = self
+            .path
+            .canonicalize()
+            .map_err(|err| anyhow::anyhow!("Failed to canonicalize {:?}: {:?}", self.path, err))?;
+        Ok(format!("--manifest-path={}", path.to_string_lossy()))
     }
 
     /// The directory path of the manifest path.
@@ -283,6 +287,17 @@ impl Manifest {
         Ok(self)
     }
 
+    /// Set empty `[workspace]` section if it does not exist.
+    ///
+    /// Ignores the `workspace` from the parent `Cargo.toml`.
+    /// This can reduce the size of the contract in some cases.
+    pub fn with_workspace(&mut self) -> Result<&mut Self> {
+        if let toml::map::Entry::Vacant(value) = self.toml.entry("workspace") {
+            value.insert(value::Value::Table(Default::default()));
+        }
+        Ok(self)
+    }
+
     /// Get mutable reference to `[profile.release]` section
     fn get_profile_release_table_mut(&mut self) -> Result<&mut value::Table> {
         let profile = self
